package dev.inmo.postssystem.core.publishing

import dev.inmo.micro_utils.repos.create
import dev.inmo.postssystem.core.post.PostId
import dev.inmo.postssystem.core.post.RegisteredPost
import dev.inmo.postssystem.core.post.repo.PostsRepo
import dev.inmo.postssystem.core.publishing.repos.*
import kotlinx.coroutines.flow.*
import kotlinx.serialization.Serializable

interface PublishingTrigger {
    val postingTriggeredFlow: SharedFlow<PublishedPost>

    suspend fun triggerPosting(
        triggerControlKey: TriggerControlKey
    ): PostId?
}

class BusinessPublishingTrigger(
    private val postsRepo: PostsRepo,
    private val publishedPostsRepo: PublishedPostsWriteRepo,
    private val publishingKeysRepo: PublishingKeysRepo
) : PublishingTrigger {
    private val _postingTriggeredFlow: MutableSharedFlow<PublishedPost> = MutableSharedFlow()
    override val postingTriggeredFlow: SharedFlow<PublishedPost> = _postingTriggeredFlow.asSharedFlow()

    override suspend fun triggerPosting(triggerControlKey: TriggerControlKey): PostId? {
        val postId = publishingKeysRepo.getPostIdByTriggerControlKey(triggerControlKey) ?: return null
        publishingKeysRepo.unsetPostTriggerControlKey(postId)

        return postsRepo.getPostById(postId) ?.let { post ->
            publishedPostsRepo.create(post).firstOrNull() ?.also {
                _postingTriggeredFlow.emit(it)
            }
        } ?.id
    }
}
