package dev.inmo.postssystem.core.publishing.repos

import com.soywiz.klock.DateTime
import dev.inmo.micro_utils.pagination.*
import dev.inmo.micro_utils.repos.*
import dev.inmo.postssystem.core.UnixMillis
import dev.inmo.postssystem.core.content.Content
import dev.inmo.postssystem.core.content.RegisteredContent
import dev.inmo.postssystem.core.post.PostId
import dev.inmo.postssystem.core.post.RegisteredPost
import dev.inmo.postssystem.core.post.repo.PostsRepo
import kotlinx.coroutines.flow.Flow
import kotlinx.serialization.Serializable
import kotlinx.serialization.Transient

typealias PublishedPostId = String
@Serializable
data class PublishedPost(
    val id: PublishedPostId,
    val post: RegisteredPost,
    val content: List<RegisteredContent>,
    private val publicationTime: UnixMillis
) {
    @Transient
    val publicationDateTime = DateTime(publicationTime)
}

interface PublishedPostsReadRepo : ReadCRUDRepo<PublishedPost, PublishedPostId> {
    suspend fun getPostPublishing(
        postId: PostId,
        pagination: Pagination,
        reversed: Boolean = false
    ): PaginationResult<PublishedPostId>
}
suspend inline fun PostId.publishedAtLeastOnce(lookIn: PublishedPostsReadRepo): Boolean {
    return lookIn.getPostPublishing(this, FirstPagePagination(1)).results.isNotEmpty()
}
suspend inline fun RegisteredPost.publishedAtLeastOnce(lookIn: PublishedPostsReadRepo): Boolean {
    return lookIn.getPostPublishing(id, FirstPagePagination(1)).results.isNotEmpty()
}
interface PublishedPostsWriteRepo : WriteCRUDRepo<PublishedPost, PublishedPostId, RegisteredPost>

interface PublishedPostsRepo : PublishedPostsReadRepo, PublishedPostsWriteRepo, CRUDRepo<PublishedPost, PublishedPostId, RegisteredPost>
