package dev.inmo.postssystem.core.publishing.repos

import dev.inmo.micro_utils.pagination.FirstPagePagination
import dev.inmo.micro_utils.repos.*
import dev.inmo.postssystem.core.post.PostId
import dev.inmo.postssystem.core.publishing.TriggerControlKey

interface ReadPublishingKeysRepo {
    suspend fun getPostIdByTriggerControlKey(
        key: TriggerControlKey
    ): PostId?
    suspend fun getTriggerControlKeyByPostId(
        postId: PostId
    ): TriggerControlKey?
}

interface WritePublishingKeysRepo {
    suspend fun setPostTriggerControlKey(
        postId: PostId,
        key: TriggerControlKey
    ): Boolean
    suspend fun unsetPostTriggerControlKey(
        postId: PostId
    ): Boolean
}

interface PublishingKeysRepo : ReadPublishingKeysRepo, WritePublishingKeysRepo

fun PublishingKeysRepo(
    keyValueRepo: KeyValueRepo<PostId, TriggerControlKey>
) = object : PublishingKeysRepo {
    override suspend fun getPostIdByTriggerControlKey(
        key: TriggerControlKey
    ): PostId? = keyValueRepo.keys(key, FirstPagePagination(1)).results.firstOrNull()

    override suspend fun getTriggerControlKeyByPostId(
        postId: PostId
    ): TriggerControlKey? = keyValueRepo.get(postId)

    override suspend fun setPostTriggerControlKey(
        postId: PostId,
        key: TriggerControlKey
    ): Boolean {
        keyValueRepo.set(postId, key)
        return true
    }

    override suspend fun unsetPostTriggerControlKey(
        postId: PostId
    ): Boolean {
        keyValueRepo.unset(postId)
        return true
    }
}
