package dev.inmo.postssystem.core.publishing.repos

import dev.inmo.micro_utils.repos.*
import dev.inmo.postssystem.core.post.PostId
import dev.inmo.postssystem.core.publishing.*

interface ReadTriggersToPostsRepo : ReadKeyValueRepo<PostId, TriggerId>
interface WriteTriggersToPostsRepo : WriteKeyValueRepo<PostId, TriggerId>
interface TriggersToPostsRepo : KeyValueRepo<PostId, TriggerId>, WriteTriggersToPostsRepo, ReadTriggersToPostsRepo

fun TriggersToPostsRepo(
    keyValueRepo: KeyValueRepo<PostId, TriggerId>,
    generator: PostKeyGenerator,
    publishingKeyReceiverGetter: PublishingKeyReceiverGetter,
    writePublishingKeysRegistrar: WritePublishingRegistrar
): TriggersToPostsRepo = object : TriggersToPostsRepo, KeyValueRepo<PostId, TriggerId> by keyValueRepo {
    override suspend fun set(toSet: Map<PostId, TriggerId>) {
        keyValueRepo.set(
            toSet.filter { (postId, triggerId) ->
                val publishingKeyReceiver = publishingKeyReceiverGetter(triggerId) ?: return@filter false
                val publishingKey = generator(postId, triggerId)
                writePublishingKeysRegistrar.registerTriggerForPost(publishingKey, postId).also {
                    if (it) {
                        publishingKeyReceiver.acceptKey(postId, publishingKey)
                    }
                }
            }
        )
    }

    override suspend fun unset(toUnset: List<PostId>) {
        toUnset.forEach {
            writePublishingKeysRegistrar.unregisterTriggerForPost(it)
        }
        keyValueRepo.unset(toUnset)
    }
}
